/**
 * Copyright (c) 2009-2011, netBout.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are PROHIBITED without prior written permission from
 * the author. This product may NOT be used anywhere and on any computer
 * except the server platform of netBout Inc. located at www.netbout.com.
 * Federal copyright law prohibits unauthorized reproduction by any means
 * and imposes fines up to $25,000 for violation. If you received
 * this code occasionally and without intent to use it, please report this
 * incident to the author by email.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package com.netbout.rest;

import com.netbout.rest.jaxb.Link;
import com.netbout.rest.jaxb.LongHelper;
import com.netbout.rest.jaxb.LongIdentity;
import com.netbout.rest.page.JaxbBundle;
import com.netbout.spi.Helper;
import com.netbout.spi.Identity;
import com.netbout.utils.Cryptor;
import com.rexsl.core.Manifests;
import com.rexsl.core.XslResolver;
import java.util.ArrayList;
import java.util.Collection;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.NewCookie;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;
import javax.xml.bind.Marshaller;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAnyElement;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlMixed;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Page.
 *
 * @author Yegor Bugayenko (yegor@netbout.com)
 * @version $Id$
 */
@XmlRootElement(name = "page")
@XmlAccessorType(XmlAccessType.NONE)
public abstract class AbstractPage implements Page {

    /**
     * Name of the user authentication cookie.
     */
    public static final String AUTH_COOKIE = "netbout";

    /**
     * Name of the message transferring cookie.
     */
    public static final String MESSAGE_COOKIE = "netbout-msg";

    /**
     * Home resource of this page.
     */
    private transient Resource home;

    /**
     * Collection of elements.
     */
    private final transient Collection elements = new ArrayList();

    /**
     * Collection of links.
     */
    private final transient Collection<Link> links = new ArrayList<Link>();

    /**
     * Initializer.
     * @param res Home of this page
     * @return This object
     */
    public final Page init(final Resource res) {
        this.home = res;
        this.link("self", this.home.uriInfo().getAbsolutePathBuilder());
        this.link("home", this.home.base());
        return this;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final Page link(final String name, final String href) {
        this.links.add(
            new Link(
                name,
                this.home.base().path(href).build()
            )
        );
        return this;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final Page link(final String name, final UriBuilder uri) {
        this.links.add(new Link(name, uri));
        return this;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final Page append(final Object element) {
        this.elements.add(element);
        if (!(element instanceof org.w3c.dom.Element)) {
            final XslResolver resolver = (XslResolver) this.home.providers()
                .getContextResolver(
                    Marshaller.class,
                    MediaType.APPLICATION_XML_TYPE
                );
            resolver.add(element.getClass());
        }
        return this;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final Page append(final JaxbBundle bundle) {
        this.append(bundle.element());
        return this;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final Response.ResponseBuilder authenticated(
        final Identity identity) {
        if (identity instanceof Helper) {
            this.append(new LongHelper((Helper) identity));
        } else {
            this.append(new LongIdentity(identity));
        }
        this.append(new JaxbBundle("auth", new Cryptor().encrypt(identity)));
        this.link("logout", "/g/out");
        this.link("start", "/s");
        this.extend();
        return Response.ok()
            .entity(this)
            .header(HttpHeaders.SET_COOKIE, this.nocookie(this.MESSAGE_COOKIE))
            .cookie(
                new NewCookie(
                    this.AUTH_COOKIE,
                    new Cryptor().encrypt(identity),
                    this.home.base().build().getPath(),
                    this.home.base().build().getHost(),
                    // @checkstyle MultipleStringLiterals (1 line)
                    Integer.valueOf(Manifests.read("Netbout-Revision")),
                    "Netbout.com logged-in user",
                    // @checkstyle MagicNumber (1 line)
                    60 * 60 * 24 * 90,
                    false
                )
            )
            .type(MediaType.TEXT_XML);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final Response.ResponseBuilder anonymous() {
        this.link("login", "/g");
        this.extend();
        return Response.ok()
            .entity(this)
            .header(HttpHeaders.SET_COOKIE, this.nocookie(this.MESSAGE_COOKIE))
            .header(HttpHeaders.SET_COOKIE, this.nocookie(this.AUTH_COOKIE))
            .type(MediaType.TEXT_XML);
    }

    /**
     * Get all elements.
     * @return Full list of injected elements
     */
    @XmlAnyElement(lax = true)
    @XmlMixed
    public final Collection<Object> getElements() {
        return this.elements;
    }

    /**
     * Get all links.
     * @return Full list of links
     */
    @XmlElement(name = "link")
    @XmlElementWrapper(name = "links")
    public final Collection<Link> getLinks() {
        return this.links;
    }

    /**
     * Get time of page generation, in nanoseconds.
     * @return Time in nanoseconds
     */
    @XmlAttribute
    public final Long getNano() {
        return System.nanoTime() - this.home.nano();
    }

    /**
     * Extend page with mandatory elements.
     */
    private void extend() {
        this.append(
            new JaxbBundle("version")
                .add("name", Manifests.read("Netbout-Version"))
                .up()
                // @checkstyle MultipleStringLiterals (1 line)
                .add("revision", Manifests.read("Netbout-Revision"))
                .up()
                .add("date", Manifests.read("Netbout-Date"))
                .up()
        );
        this.append(new JaxbBundle("message", this.home.message()));
    }

    /**
     * Create header that cleans cookie with the given name.
     * @param name Name of the cookie
     * @return Value of the HTTP header
     */
    private String nocookie(final String name) {
        return String.format(
            // @checkstyle LineLength (1 line)
            "%s=deleted;Domain=.%s;Path=/%s;Expires=Thu, 01-Jan-1970 00:00:01 GMT",
            name,
            this.home.base().build().getHost(),
            this.home.httpServletRequest().getContextPath()
        );
    }

}
